/* ******************************************* */
/* filename InOut.java
 * contains all classes and methods needed for loading and saving 
 * a grammar
 */

package lgrammar;

import java.io.*;
import java.awt.*;

class InOut {

	private static final char newline = '\n';
	private static final String ENDE = "Ende";
	
	private static Frame frame = new Frame("Don't need you"); // it's just
					// needed to open a FileDialog
	private static FileDialog 
	  FDLoad = null, // FileDialog to open a grammar
	  FDSave = null; // FileDialog to save a grammar
	
	/* ****************************************************** */
	/*           S A V E  G R A M M A R			  */
  	/* ****************************************************** */
  

	public static void save(Xui xui) throws InOutException{
		RandomAccessFile file = null;
		String file_str = null;
		StringBuffer sb  = new StringBuffer();
		Color[] color =	null;
		int proj_x = xui.projection.get_x(),
		    proj_y = xui.projection.get_y(),
		    proj_a = xui.projection.get_a(),
		    hint = 0, 
		    i = 0;
		double hdouble = 0;
		// ask the user in which file the grammar is to save
		FDSave = new FileDialog(frame,  "Lgrammar Save", FileDialog.SAVE);
		FDSave.show();
	
		// save it
		try{
			file_str = FDSave.getDirectory() + FDSave.getFile(); 
			if (file_str == null) throw 
				new InOutException(InOutException.e1);
			file = new RandomAccessFile(file_str, "rw");
			// save colors
			hint = xui.colors.get_palettetype();
			if (hint < Colors.OWN) {
				sb.append(hint);
				sb.append(newline);
			} else {
				sb.append(hint);
				sb.append(newline);
				color = xui.colors.get_Color();
				for (i=0; i < Colors.COLOR_NUM; i++){
					sb.append(color[i].getRed());
					sb.append(newline);
					sb.append(color[i].getGreen());
					sb.append(newline);
					sb.append(color[i].getBlue());
					sb.append(newline);
				}
			}
			// save projection
			sb.append(proj_x);
			sb.append(newline);
			sb.append(proj_y);
			sb.append(newline);
			sb.append(proj_a);
			sb.append(newline);
			// save TextFields
			// the TextFields may be changed since the last 
			// initalisation, so it's better to check them
			textf_int(xui.t_posx, 100, sb);
			textf_int(xui.t_posy, 100, sb);
			textf_int(xui.t_posz, 0,   sb);
			textf_double(xui.t_angle, 90 , sb);
			textf_int(xui.t_length, 90, sb);
			textf_double(xui.t_scale, 4,   sb);
			textf_double(xui.t_headx, 0 , sb);
			textf_double(xui.t_heady, 0 , sb);
			textf_double(xui.t_headz, 0 , sb);
			textf_double(xui.t_leftx, -1 , sb);
			textf_double(xui.t_lefty, 0 , sb);
			textf_double(xui.t_leftz, 0 , sb);
			textf_double(xui.t_upx, 0 , sb);
			textf_double(xui.t_upy, 0 , sb);
			textf_double(xui.t_upz, 1 , sb);
			// finally read the grammar_string
			sb.append(xui.grammartxt.getText());
			// put all in the file
			file.writeBytes(new String(sb));
			// to recognize the end
			file.writeBytes("\n" + ENDE + "\n"); // the last 
			    //newline is urgently needed for loading from urls
			file.close();
		} catch (IOException e) {
			throw new InOutException(InOutException.e1);
		}

	}


	// METHODS TO READ OUT OF TEXRFIELDS

	private static void textf_int(TextField t, int def, StringBuffer sb) {
		// appends the context of t to stringBuffer and adds a newline
		int a = def;
		try { 
			a = Integer.parseInt(t.getText());
	    	} catch (NumberFormatException e) {
			a = def;
		 }
		sb.append(a);
		sb.append(newline);
	}
	private static void textf_double(TextField t, double def, 
	   StringBuffer sb){
	 // appends the context of t and newline to sb!!
		double a = def;
		try {
			a = Double.valueOf(t.getText()).doubleValue();
	   	} catch(NumberFormatException e) {
			a = def;
		}
		sb.append(a);
		sb.append(newline);
	}
		
	/* ********************************************** */
	
	// METHODS FOR LOAD A GRAMMAR

	public static void load(Xui xui) throws InOutException {

		RandomAccessFile rfile = null;
		String  line = null,
			filename = null;
		long 	pointer= 0,
			length = 0;
		StringBuffer sb = new StringBuffer();
		char[] all = null;
		char c;
		Color[] color;
		int j = 0,
		    hint = 0; // for type convertions
		
		try{
			FDLoad = new FileDialog(frame,  "Lgrammar Load", FileDialog.LOAD);
			FDLoad.show();
			filename = FDLoad.getDirectory() + FDLoad.getFile();
			if (filename == null){
				throw new InOutException(InOutException.e2);
			}
			rfile = new RandomAccessFile(filename, "r");

			// read color
			line = rfile.readLine();
			hint = Integer.parseInt(line);
			if(hint < Colors.OWN){
				xui.colors.set_Color(hint);
			} else {
			  int r, g, b;
			  color = new Color[Colors.COLOR_NUM];
			  for (j=0; j<Colors.COLOR_NUM; j++){
				line = rfile.readLine();
				r = Integer.parseInt(line);
				line = rfile.readLine();
				g = Integer.parseInt(line);
				line = rfile.readLine();
				b = Integer.parseInt(line);
				color[j] = new Color(r, g, b);
			  }
			  xui.colors.set_Color(color);
			}
		 	// set projection
			int x, y, a;
			line = rfile.readLine();
			x = Integer.parseInt(line);
			line = rfile.readLine();
			y = Integer.parseInt(line);
			line = rfile.readLine();
			a = Integer.parseInt(line);
			xui.projection.set_values(x,y,a);
			// set the textfields
			line = rfile.readLine();
			xui.t_posx.setText(line);
			line = rfile.readLine();
			xui.t_posy.setText(line);
			line = rfile.readLine();
			xui.t_posz.setText(line);
			line = rfile.readLine();
			xui.t_angle.setText(line);
			line = rfile.readLine();
			xui.t_length.setText(line);
			line = rfile.readLine();
			xui.t_scale.setText(line);
			line = rfile.readLine();
			xui.t_headx.setText(line);
			line = rfile.readLine();
			xui.t_heady.setText(line);
			line = rfile.readLine();
			xui.t_headz.setText(line);
			line = rfile.readLine();
			xui.t_leftx.setText(line);
			line = rfile.readLine();
			xui.t_lefty.setText(line);
			line = rfile.readLine();
			xui.t_leftz.setText(line);
			line = rfile.readLine();
			xui.t_upx.setText(line);
			line = rfile.readLine();
			xui.t_upy.setText(line);
			line = rfile.readLine();
			xui.t_upz.setText(line);	
			// last read grammar
			sb = new StringBuffer();
			line = rfile.readLine();
			sb.append(line);
			line = rfile.readLine(); 
			while(! line.equalsIgnoreCase(ENDE)){
			    sb.append(newline);
			    sb.append(line);
			    line = rfile.readLine();
			}
			xui.grammartxt.setText(new String(sb));
			rfile.close();
		} catch (IOException e){
			throw new InOutException(e.getMessage());
		} catch (Exception e){
			throw new InOutException(e.getMessage());
		}
	}

}


class InOutException extends Exception{
	public static final String
		e1 = "Indicate File not found",
		e2 = "Couldn't load the grammar";
	// constructores
	public InOutException(String s) {	
		super(s);
	}
}
